import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { CommonModule } from '@angular/common';
import Parse from 'parse';
import { CompUploadComponent } from '../../../../app/comp-upload/comp-upload.component';
import { NzRadioModule } from 'ng-zorro-antd/radio';
import { CommonCompModule } from '../../../../services/common.modules';
import { NzMessageService } from 'ng-zorro-antd/message';
interface link {
  url: string;
  username?: string;
  password?: string;
}
@Component({
  selector: 'app-upload-collect',
  templateUrl: './upload-collect.component.html',
  styleUrls: ['./upload-collect.component.scss'],
  standalone: true,
  imports: [CommonCompModule, CommonModule, CompUploadComponent, NzRadioModule],
})
export class UploadCollectComponent implements OnInit {
  @Input('review') review: boolean = false; //只读
  @Input('eduTextbookVolume') eduTextbookVolume: Parse.Object | undefined;
  @Output() save: EventEmitter<any> = new EventEmitter<any>();

  collectFiles: Array<any> = []; //纸质教材PDF文件列表
  collectDigitFiles: Array<any> = []; //数字资源PDF文件列表
  collectLink: link = {
    url: '',
    username: '',
    password: '',
  }; //链接、账号密码

  radioValue: string = '上传文件';

  constructor(private msg: NzMessageService) {}

  ngOnInit() {
    this.collectFiles = this.eduTextbookVolume?.get('collectFiles');
    this.collectDigitFiles = this.eduTextbookVolume?.get('collectDigitFiles');
    this.collectLink = this.eduTextbookVolume?.get('collectLink');
    this.radioValue = this.eduTextbookVolume?.get('collectCheck');
  }
  upload(e: any, type: string) {
    console.log('上传材料发生改变');
    console.log(e);
    if (type == 'collectFiles' || type == 'collectDigitFiles') {
      this[type] = e;
    }
  }
  async submitForm(type: string) {
    if (
      this.eduTextbookVolume?.get('carrierShape') == '纸质教材' ||
      this.eduTextbookVolume?.get('carrierShape') == '纸质教材附带电子资源'
    ) {
      let upload = this.collectFiles?.some((item) => item.url);
      // console.log(upload);
      if (!upload && type == 'sbmit') {
        // this.msg.warning('请上传纸质教材PDF文件');
        return;
      }
      this.eduTextbookVolume?.set('collectFiles', this.collectFiles);
    }
    if (
      this.eduTextbookVolume?.get('carrierShape') == '数字教材' ||
      this.eduTextbookVolume?.get('carrierShape') == '纸质教材附带电子资源'
    ) {
      let upload = this.collectDigitFiles?.some((item) => item.url);
      // console.log(upload);
      if (this.radioValue == '上传文件' && !upload && type == 'sbmit') {
        // this.msg.warning('请上传数字文件PDF');
        return;
      }
      this.eduTextbookVolume?.set('collectDigitFiles', this.collectDigitFiles);
      this.eduTextbookVolume?.set('collectCheck', this.radioValue);
      if (
        type == 'sbmit' &&
        ((this.radioValue == '链接' && !this.collectLink?.url) ||
          (this.radioValue == '链接和账号密码' &&
            (!this.collectLink?.url ||
              !this.collectLink?.username ||
              !this.collectLink?.password)))
      ) {
        // this.msg.warning('请填写完整信息');
        return;
      }
      this.eduTextbookVolume?.set('collectLink', this.collectLink);
    }
    /* 100：已上传 200：已提交*/
    this.eduTextbookVolume?.set(
      'collectStatus',
      type == 'sbmit' ? '200' : '100'
    );
    await this.eduTextbookVolume?.save();
    return true;
  }

  openUrl(url: string) {
    window.open(url);
  }
}
